/* config.v - Configuration values for the 9086 CPU and the system around it

   This file is part of the 9086 project.

   Copyright (c) 2023 Efthymios Kritikos

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/** Runtime Verbosity **/
//`define DEBUG_REG_WRITES
//`define DEBUG_PC_ADDRESS
//`define DEBUG_DATA_READ_WRITES

/** OUTPUT **/
/* These are usually set at build time*/
//`define CALCULATE_IPC
//`define OTUPUT_JSON_STATISTICS

/** Optimisations **/

/* Enables the ability to check if an instruction in the input
 * buffer is completed and mark it ready instead of always waiting
 * for the maximum instruction size worth of bytes */
`define EARLY_VALID_INSTRUCTION

/*********** CURRENTLY DOESN'T WORK *************/
/* Enables the ability in BIU to pre-decode two instructions, one after the other in memory*/
//`define DOUBLE_INSTRUCTION_LOAD
/************************************************/

/* Size is in powers of two with minimal 3.
 *   3 : 8 Bytes
 *   4 : 16 Bytes
 *   5 : 32 Bytes
 *   . :  ... */
`define L1_CACHE_SIZE 4

//// These are usually set at build time
//
// /* This is the "virtual" synthesised ram, so for example on an FPGA
//  * This would be made inside the fabric of the fpga. */
// `define BUILTIN_RAM 512
// ////// ATTENTION: PLEASE IF BUILTIN_RAM DOESN'T COVER THE ENTIRE (CURRENTLY)
// // 16 BIT RANGE, I.E. ISN'T 32768, PLEASE SET THE FOLLOWING FLAG
// `define NOT_FULL


/********** Internal **********/
`ifdef SYNTHESIS
	`undef DEBUG_REG_WRITES
	`undef DEBUG_PC_ADDRESS
	`undef DEBUG_DATA_READ_WRITES
	`undef CALCULATE_IPC
	`undef OTUPUT_JSON_STATISTICS
`endif

`ifdef OTUPUT_JSON_STATISTICS
	`define CALCULATE_IPC
`endif

`ifdef DOUBLE_INSTRUCTION_LOAD
	/*Needed for DOUBLE_INSTRUCTION_LOAD*/
	`define EARLY_VALID_INSTRUCTION
`endif

`ifdef EARLY_VALID_INSTRUCTION
	`define INCLUDE_EARLY_CALC_CIRUIT
	`define EARLY_VALID_INSTRUCTION_ 1
`else
	`define EARLY_VALID_INSTRUCTION_ 0
`endif

`ifdef CALCULATE_IPC
	`define INCLUDE_EARLY_CALC_CIRUIT
`endif
